# -*- coding: utf-8 -*-

"""Tests for date generator."""

import datetime
from fauxfactory import gen_date
from fauxfactory.constants import MAX_YEARS, MIN_YEARS
from sys import version_info

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestDates(unittest.TestCase):
    """Test date generator."""

    def test_gen_date_1(self):
        """
        @Test: Create a date with no arguments
        @Feature: Date Generator
        @Assert: Date is created with random values.
        """
        result = gen_date()
        self.assertTrue(
            isinstance(result, datetime.date),
            "Data is not instance of datetime.date.")

    def test_gen_date_2(self):
        """
        @Test: Create a date with only min_date
        @Feature: Date Generator
        @Assert: Date should be created and not be before minimum date
        """

        # Today is...
        today = datetime.date.today()
        # Five days ago
        min_date = today - datetime.timedelta(5)

        for turn in range(10):
            result = gen_date(min_date=min_date)
            self.assertTrue(result >= min_date)

    def test_gen_date_3(self):
        """
        @Test: Create a date with only max_date
        @Feature: Date Generator
        @Assert: Date should be created and not be after maximum date
        """

        # Today is...
        today = datetime.date.today()
        # Five days into the future
        max_date = today + datetime.timedelta(5)

        for turn in range(10):
            result = gen_date(max_date=max_date)
            self.assertTrue(result <= max_date)

    def test_gen_date_4(self):
        """
        @Test: Create a date with both arguments
        @Feature: Date Generator
        @Assert: Date should be created and fall within specified range
        """

        # Today is...
        today = datetime.date.today()
        # Five days ago
        min_date = today - datetime.timedelta(5)
        # Five days into the future
        max_date = today + datetime.timedelta(5)

        for turn in range(10):
            result = gen_date(
                min_date=min_date,
                max_date=max_date
            )
            self.assertTrue(result >= min_date)
            self.assertTrue(result <= max_date)

    def test_gen_date_5(self):
        """
        @Test: Create a date with min_date == 'None'
        @Feature: Date Generator
        @Assert: Date should be created and fall before maximum date
        """

        # min_date for the platform
        min_date = (datetime.date.today() -
                    datetime.timedelta(365 * MIN_YEARS))
        # max_date = min_date + 1 year
        max_date = min_date + datetime.timedelta(365 * 1)

        for turn in range(20):
            result = gen_date(
                min_date=None,
                max_date=max_date
            )
            self.assertTrue(result.year <= max_date.year)
            self.assertTrue(result.year >= min_date.year)

    def test_gen_date_6(self):
        """
        @Test: Create a date with max_date == 'None'
        @Feature: Date Generator
        @Assert: Date should be created and fall after minimum date
        """

        # max_date for the platform
        max_date = (datetime.date.today() +
                    datetime.timedelta(365 * MAX_YEARS))
        # min_date  = max_date - 1 year
        min_date = max_date - datetime.timedelta(365 * 1)

        for turn in range(20):
            result = gen_date(
                min_date=min_date,
                max_date=None
            )
            self.assertTrue(result.year <= max_date.year, result)
            self.assertTrue(result.year >= min_date.year, result)

    def test_gen_date_7(self):
        """
        @Test: Create a date with specific date ranges
        @Feature: Date Generator
        @Assert: Date should be created and fall inside the date range
        """

        # min_date for the platform
        min_date = (datetime.date.today() -
                    datetime.timedelta(365 * MIN_YEARS))
        # max_date for the platform
        max_date = (datetime.date.today() +
                    datetime.timedelta(365 * MAX_YEARS))

        for turn in range(20):
            result = gen_date(
                min_date=min_date,
                max_date=max_date
            )
            self.assertTrue(result.year <= max_date.year, result)
            self.assertTrue(result.year >= min_date.year, result)

    def test_gen_date_8(self):
        """
        @Test: Create a date with non-Date arguments
        @Feature: Date Generator
        @Assert: Date should not be created due to value error
        """

        with self.assertRaises(ValueError):
            gen_date(
                min_date='',
                max_date=''
            )

    def test_gen_date_9(self):
        """
        @Test: Create a date with non-Date arguments
        @Feature: Date Generator
        @Assert: Date should not be created due to value error
        """

        with self.assertRaises(ValueError):
            gen_date(
                min_date='abc',
                max_date='def'
            )

    def test_gen_date_10(self):
        """
        @Test: Create a date with non-Date arguments
        @Feature: Date Generator
        @Assert: Date should not be created due to value error
        """

        with self.assertRaises(ValueError):
            gen_date(
                min_date=1,
                max_date=1
            )

    def test_gen_date_11(self):
        """
        @Test: Create a date with non-Date arguments
        @Feature: Date Generator
        @Assert: Date should not be created due to value error
        """

        with self.assertRaises(ValueError):
            gen_date(
                min_date=(1,),
                max_date=(2, 3, 4)
            )

    def test_gen_date_12(self):
        """
        @Test: Create a date with non-Date arguments
        @Feature: Date Generator
        @Assert: Date should not be created due to value error
        """

        with self.assertRaises(ValueError):
            gen_date(
                min_date=['a', 'b'],
                max_date=['c', 'd', 'e']
            )

    def test_gen_date_13(self):
        """
        @Test: Create a date with min_date > max_date
        @Feature: Date Generator
        @Assert: Date should not be created due to assertion error
        """

        # Today is...
        today = datetime.date.today()
        # Five days into the future
        min_date = today + datetime.timedelta(5)

        with self.assertRaises(AssertionError):
            gen_date(
                min_date=min_date,
                max_date=today
            )

    def test_gen_date_14(self):
        """
        @Test: max-date must be a Date type
        @Feature: Date Generator
        @Assert: Date should not be created due to value error
        """

        with self.assertRaises(ValueError):
            gen_date(
                min_date=datetime.date.today(),
                max_date='foo'
            )



# -*- coding: utf-8 -*-

from sys import version_info

from fauxfactory import _check_validation

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest
    if version_info[0] == 2:
        from mock import Mock
    else:
        from unittest.mock import Mock


@_check_validation
def decorated_f():
    return 'not a number'


class CheckValidationTestCase(unittest.TestCase):
    """_check_validation decorator tests"""

    def test_no_validator_defined(self):
        """Check result value of decorated function is returned when no
        validator is provided
        """
        self.assertEqual('not a number', decorated_f())

    def test_validator_defined_but_default_is_none(self):
        """Check defining validator but not default raises an error"""
        self.assertRaises(ValueError, decorated_f, validator=lambda _: True)

    def test_regex(self):
        """Check regex validation when validator is a string"""
        self.assertEqual(
            'my default', decorated_f(validator=r'\d.*', default='my default'))
        self.assertEqual(
            'not a number', decorated_f(validator=r'.*', default='my default'))

    def test_callable(self):
        """Check validation when validator is a callable"""
        callable = Mock(return_value=False)

        # Default of 10 unsuccessful tries
        self.assertEqual(
            'my default',
            decorated_f(validator=callable, default='my default')
        )
        callable.assert_called_with('not a number')
        self.assertEqual(10, callable.call_count)

        # 1 unsuccessful try
        callable.reset_mock()
        self.assertEqual(
            'my default',
            decorated_f(validator=callable, default='my default', tries=1)
        )
        callable.assert_called_once_with('not a number')

        # 1 successful try
        callable.reset_mock()
        callable.return_value = True
        self.assertEqual(
            'not a number',
            decorated_f(validator=callable, default='my default', tries=10)
        )
        callable.assert_called_once_with('not a number')



# -*- coding: utf-8 -*-

"""Tests for all number generators."""

from fauxfactory import (
    gen_integer,
    gen_negative_integer,
    gen_positive_integer,
)

import sys
if sys.version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestNumbers(unittest.TestCase):
    """Test number generators."""

    def test_gen_integer_1(self):
        """
        @Test: Create a random integer with no range limits
        @Feature: Numbers Generator
        @Assert: A random integer is created
        """

        if sys.version_info[0] == 2:
            integer_types = (int, long,)  # flake8:noqa
        else:
            integer_types = (int,)
        result = gen_integer()
        self.assertTrue(
            isinstance(result, integer_types),
            "A valid integer was not generated."
        )

    def test_gen_integer_2(self):
        """
        @Test: Create a random integer with set minimum limit
        @Feature: Numbers Generator
        @Assert: Integer is created and greater than minimum
        """

        try:
            # Change system max int to a smaller number
            old_sys_maxsize = sys.maxsize
            sys.maxsize = 5

            for turn in range(10):
                result = gen_integer(min_value=1)
                self.assertTrue(
                    result <= sys.maxsize, "Integer is greater than max_value"
                )
                self.assertTrue(
                    result >= 1, "Integer is less than specified minimum"
                )
        finally:
            # Reset system max int back to original value
            sys.maxsize = old_sys_maxsize

    def test_gen_integer_3(self):
        """
        @Test: Create a random integer with set maximum limit
        @Feature: Numbers Generator
        @Assert: Integer is created and less than maximum value
        """

        try:
            # Change system max int to a smaller number
            old_sys_maxsize = sys.maxsize
            sys.maxsize = 5
            min_value = - sys.maxsize - 1

            for turn in range(10):
                result = gen_integer(max_value=1)
                self.assertTrue(
                    result >= min_value, "Integer is less than min_value"
                )
                self.assertTrue(
                    result <= 1, "Integer is greater than specified maximum"
                )
        finally:
            # Reset system max int back to original value
            sys.maxsize = old_sys_maxsize

    def test_gen_integer_4(self):
        """
        @Test: Create a random integer with set min/max limits
        @Feature: Numbers Generator
        @Assert: An integer is created and falls within the specified range
        """

        for turn in range(10):
            result = gen_integer(
                min_value=1, max_value=3)
            self.assertTrue(
                result >= 1, "Integer is less than min_value"
            )
            self.assertTrue(
                result <= 3, "Integer is greater than specified maximum"
            )

    def test_gen_integer_5(self):
        """
        @Test: Create a random integer with disallowed minimum limit
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        # This is lower than allowed platform minimum
        low_min = - sys.maxsize - 2

        with self.assertRaises(ValueError):
            gen_integer(min_value=low_min)

    def test_gen_integer_6(self):
        """
        @Test: Create a random integer with disallowed maximum limit
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        # This is greater than allowed platform maximum
        high_max = sys.maxsize + 1

        with self.assertRaises(ValueError):
            gen_integer(max_value=high_max)

    def test_gen_integer_7_0(self):
        """
        @Test: Create a random integer using empty strings as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(min_value='')

    def test_gen_integer_7_1(self):
        """
        @Test: Create a random integer using empty strings as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(max_value='')

    def test_gen_integer_7_2(self):
        """
        @Test: Create a random integer using empty strings as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(min_value='', max_value='')

    def test_gen_integer_8_0(self):
        """
        @Test: Create a random integer using whitespace as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(min_value=' ')

    def test_gen_integer_8_1(self):
        """
        @Test: Create a random integer using whitespace as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(max_value=' ')

    def test_gen_integer_8_2(self):
        """
        @Test: Create a random integer using whitespace as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(min_value=' ', max_value=' ')

    def test_gen_integer_9_0(self):
        """
        @Test: Create a random integer using alpha strings as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(min_value='a')

    def test_gen_integer_9_1(self):
        """
        @Test: Create a random integer using alpha strings as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(max_value='a')

    def test_gen_integer_9_2(self):
        """
        @Test: Create a random integer using alpha strings as args
        @Feature: Numbers Generator
        @Assert: An integer number is not created due to value error
        """

        with self.assertRaises(ValueError):
            gen_integer(min_value='a', max_value='b')

    def test_gen_positive_integer_1(self):
        """
        @Test: Create a random positive integer
        @Feature: Numbers Generator
        @Assert: A positive number is created
        """

        result = gen_positive_integer()

        self.assertTrue(result >= 0, "Generated integer is not positive")

    def test_gen_negative_integer_1(self):
        """
        @Test: Create a random negative integer
        @Feature: Numbers Generator
        @Assert: A negative number is created
        """

        result = gen_negative_integer()

        self.assertTrue(result <= 0, "Generated integer is not negative")



# -*- coding: utf-8 -*-

"""Tests for ipaddr generator."""

from sys import version_info
from fauxfactory import gen_ipaddr

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestIpaddr(unittest.TestCase):
    """Test ipaddr generator."""

    def test_gen_ipv4_1(self):
        """
        @Test: Generate a 3 group IPv4 address
        @Feature: IPAddr Generator
        @Assert: A 3-group IPv4 address is generated (addr
                 will always end with a \'.0\')
        """

        result = gen_ipaddr(ip3=True)
        self.assertTrue(
            result.split(".")[-1] == '0',
            "Did not generate a 3-group IPv4 addrss")

    def test_gen_ipv4_2(self):
        """
        @Test: Generate a 4 group IPv4 address
        @Feature: IPAddr Generator
        @Assert: A 4-group IPv4 address is generated
        """

        result = gen_ipaddr()
        self.assertTrue(
            len(result.split(".")) == 4,
            "Did not generate a 4-group IPv4 addrss")

    def test_gen_ipv4_3(self):
        """
        @Test: Generate a 4 group IPv4 address
        @Feature: IPAddr Generator
        @Assert: A 4-group IPv4 address is generated
        """

        result = gen_ipaddr(ip3=False)
        self.assertTrue(
            len(result.split(".")) == 4,
            "Did not generate a 4-group IPv4 addrss")

    def test_gen_ipv4_4(self):
        """
        @Test: Generate a 4 group IPv4 address
        @Feature: IPAddr Generator
        @Assert: A 4-group IPv4 address is generated
        """

        result = gen_ipaddr(ip3=False, ipv6=False)
        self.assertTrue(
            len(result.split(".")) == 4,
            "Did not generate a 4-group IPv4 addrss")

    def test_gen_ipv4_5(self):
        """
        @Test: Generate a 4 group IPv4 address with good prefix
        @Feature: IPAddr Generator
        @Assert: A 4-group IPv4 address is generated
        """

        result = gen_ipaddr(ip3=False, ipv6=False, prefix=[10])
        self.assertTrue(
            len(result.split(".")) == 4,
            "Did not generate a 4-group IPv4 addrss")
        self.assertTrue(result.startswith("10."))

    def test_gen_ipv4_6(self):
        """
        @Test: Generate a 4 group IPv4 address with good prefix
        @Feature: IPAddr Generator
        @Assert: A 4-group IPv4 address is generated
        """

        result = gen_ipaddr(ip3=False, ipv6=False, prefix=[10, 10])
        self.assertTrue(
            len(result.split(".")) == 4,
            "Did not generate a 4-group IPv4 addrss")
        self.assertTrue(result.startswith("10.10."))

    def test_gen_ipv4_7(self):
        """
        @Test: Generate a 4 group IPv4 address with good prefix
        @Feature: IPAddr Generator
        @Assert: A 4-group IPv4 address is generated
        """

        result = gen_ipaddr(ip3=False, ipv6=False, prefix=[10, 10, 10])
        self.assertTrue(
            len(result.split(".")) == 4,
            "Did not generate a 4-group IPv4 addrss")
        self.assertTrue(result.startswith("10.10.10."))

    def test_gen_ipv4_8(self):
        """
        @Test: Generate a 4 group IPv4 address with prefix disabling randomness
        @Feature: IPAddr Generator
        @Assert: An exception is raised
        """

        with self.assertRaises(ValueError):
            gen_ipaddr(ip3=False, ipv6=False, prefix=[10, 10, 10, 10])

    def test_gen_ipv4_9(self):
        """
        @Test: Generate a 4 group IPv4 address with prefix too long
        @Feature: IPAddr Generator
        @Assert: An exception is raised
        """

        with self.assertRaises(ValueError):
            gen_ipaddr(ip3=False, ipv6=False, prefix=[10, 10, 10, 10, 10])

    def test_gen_ipv4_10(self):
        """
        @Test: Generate a 3 group IPv4 address with good prefix
        @Feature: IPAddr Generator
        @Assert: A 3-group IPv4 address is generated
        """

        result = gen_ipaddr(ip3=True, ipv6=False, prefix=[10, 10])
        self.assertTrue(
            len(result.split(".")) == 4,
            "Did not generate a 4-group IPv4 addrss")
        self.assertTrue(result.startswith("10.10."))
        self.assertTrue(result.endswith(".0"))

    def test_gen_ipv4_11(self):
        """
        @Test: Generate a 3 group IPv4 address with prefix disabling randomness
        @Feature: IPAddr Generator
        @Assert: An exception is raised
        """

        with self.assertRaises(ValueError):
            gen_ipaddr(ip3=True, ipv6=False, prefix=[10, 10, 10])

    def test_gen_ipv4_12(self):
        """
        @Test: Generate a 3 group IPv4 address with prefix too long
        @Feature: IPAddr Generator
        @Assert: An exception is raised
        """

        with self.assertRaises(ValueError):
            gen_ipaddr(ip3=True, ipv6=False, prefix=[10, 10, 10, 10])

    def test_gen_ipv6_1(self):
        """
        @Test: Generate a IPv6 address
        @Feature: IPAddr Generator
        @Assert: A IPv6 address is generated
        """

        result = gen_ipaddr(ipv6=True)
        self.assertTrue(
            len(result.split(":")) == 8,
            "Did not generate a IPv6 addrss")

    def test_gen_ipv6_2(self):
        """
        @Test: Generate a IPv6 address
        @Feature: IPAddr Generator
        @Assert: A IPv6 address is generated
        """

        result = gen_ipaddr(ip3=True, ipv6=True)
        self.assertTrue(
            len(result.split(":")) == 8,
            "Did not generate a IPv6 addrss")

    def test_gen_ipv6_3(self):
        """
        @Test: Generate a IPv6 address with custom prefix
        @Feature: IPAddr Generator
        @Assert: A IPv6 address is generated
        """

        result = gen_ipaddr(ipv6=True, prefix=["e2d3"])
        self.assertTrue(
            len(result.split(":")) == 8,
            "Did not generate a IPv6 addrss")
        self.assertTrue(result.startswith("e2d3:"))

    def test_gen_ipv6_4(self):
        """
        @Test: Generate a IPv6 address with custom (very long) prefix
        @Feature: IPAddr Generator
        @Assert: A IPv6 address is generated
        """

        prefix = 7 * ["e2d3"]
        result = gen_ipaddr(ipv6=True, prefix=prefix)
        self.assertTrue(
            len(result.split(":")) == 8,
            "Did not generate a IPv6 addrss")
        self.assertTrue(result.startswith(":".join(prefix)))

    def test_gen_ipv6_5(self):
        """
        @Test: Generate a IPv6 address with too long prefix
        @Feature: IPAddr Generator
        @Assert: ValueError raised
        """

        prefix = 8 * ["e2d3"]
        with self.assertRaises(ValueError):
            gen_ipaddr(ipv6=True, prefix=prefix)

    def test_gen_ipv6_6(self):
        """
        @Test: Generate a IPv6 address with even longer prefix
        @Feature: IPAddr Generator
        @Assert: ValueError raised
        """

        prefix = 9 * ["e2d3"]
        with self.assertRaises(ValueError):
            gen_ipaddr(ipv6=True, prefix=prefix)



# -*- coding: utf-8 -*-

"""Tests for Lorem Ipsum generator."""

import random
from sys import version_info
from fauxfactory import gen_iplum
from fauxfactory.constants import LOREM_IPSUM_TEXT

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestLoremIpsum(unittest.TestCase):
    """
    Test lorem ipsum generator
    """

    def test_gen_loremipsum_1(self):
        """
        @Test: Create a complete lorem ipsum string
        @Feature: Lorem Ipsum Generator
        @Assert: Complete lorem ipsum value that starts with 'Lorem ipsum'
        """

        result = gen_iplum()
        self.assertEqual(
            result,
            LOREM_IPSUM_TEXT, "Generated text is not Lorem Ipsum")
        self.assertTrue(
            result.startswith('Lorem ipsum'),
            "Generated string does not start with Lorem Ipsum"
        )

    def test_gen_loremipsum_2(self):
        """
        @Test: Create a lorem ipsum string with fixed number of words
        @Feature: Lorem Ipsum Generator
        @Assert: lorem ipsum value has exact number of words
        """

        for i in range(20):
            length = random.randint(1, 500)
            result = gen_iplum(words=length)
            self.assertEqual(
                len(result.split()),
                length,
                "Generated string does not have the correct number of words")

    def test_gen_loremipsum_3(self):
        """
        @Test: Create a lorem ipsum string with fixed number of paragraphs
        @Feature: Lorem Ipsum Generator
        @Assert: length lorem ipsum value has exact number of paragraphs
        """

        for i in range(20):
            length = random.randint(1, 20)
            result = gen_iplum(paragraphs=length)
            self.assertEqual(
                len(result.split('\n')),
                length,
                "Generated string does not have the correct number"
                " of paragraphs")

    def test_gen_loremipsum_4(self):
        """
        @Test: Create a lorem ipsum string with zero words
        @Feature: Lorem Ipsum Generator
        @Assert: Complete lorem ipsum value is returned
        """

        result = gen_iplum(words=0)
        self.assertEqual(
            result,
            LOREM_IPSUM_TEXT, "Generated text is not Lorem Ipsum")

    def test_gen_loremipsum_5(self):
        """
        @Test: Create a lorem ipsum string with zero paragraphs
        @Feature: Lorem Ipsum Generator
        @Assert: Raises ValueError
        """

        with self.assertRaises(ValueError):
            gen_iplum(paragraphs=0)

    def test_gen_loremipsum_6(self):
        """
        @Test: Create a lorem ipsum string with 1 word and 0 paragragh
        @Feature: Lorem Ipsum Generator
        @Assert: Raise ValueError
        """

        with self.assertRaises(ValueError):
            gen_iplum(words=1, paragraphs=0)

    def test_gen_loremipsum_7(self):
        """
        @Test: Create a lorem ipsum string with 1 word and 1 paragragh
        @Feature: Lorem Ipsum Generator
        @Assert: Generated string has 1 word in 1 paragraph
        """

        result = gen_iplum(words=1, paragraphs=1)
        self.assertEqual(len(result.split()), 1, "String is not 1-word long")
        self.assertEqual(
            len(result.split()), 1, "String is not 1-paragraph long")

    def test_gen_loremipsum_8(self):
        """
        @Test: Create a lorem ipsum string with non-integer words
        @Feature: Lorem Ipsum Generator
        @Assert: Raises ValueError
        """

        with self.assertRaises(ValueError):
            gen_iplum(words='a')

    def test_gen_loremipsum_9(self):
        """
        @Test: Create a lorem ipsum string with non-integer paragraphs
        @Feature: Lorem Ipsum Generator
        @Assert: Raises ValueError
        """

        with self.assertRaises(ValueError):
            gen_iplum(paragraphs='a')

    def test_gen_loremipsum_10(self):
        """
        @Test: Create a lorem ipsum string with random words/paragraphs
        @Feature: Lorem Ipsum Generator
        @Assert: lorem ipsum value has exact number of words/paragraphs
        """

        for i in range(20):
            words = random.randint(1, 500)
            paragraphs = random.randint(1, 500)
            result = gen_iplum(
                words=words, paragraphs=paragraphs)
            self.assertEqual(
                len(result.split('\n')),
                paragraphs,
                ("Generated string does not have the correct number"
                 "of paragraphs"))
            for sentence in result.split('\n'):
                self.assertEqual(
                    len(sentence.split()),
                    words,
                    ("Generated string does not have the correct"
                     "number of words"))



# -*- coding: utf-8 -*-
"""Generate random data for your tests."""

import datetime
import random
import re
import string
import sys
import unicodedata
import uuid
import warnings
from collections import Iterable
from functools import wraps

from fauxfactory.constants import (
    HTML_TAGS, LOREM_IPSUM_TEXT,
    MAX_YEARS, MIN_YEARS,
    SCHEMES, SUBDOMAINS, TLDS, VALID_NETMASKS
)

__all__ = (
    'gen_alpha',
    'gen_alphanumeric',
    'gen_boolean',
    'gen_choice',
    'gen_cjk',
    'gen_cyrillic',
    'gen_date',
    'gen_datetime',
    'gen_email',
    'gen_html',
    'gen_integer',
    'gen_ipaddr',
    'gen_iplum',
    'gen_latin1',
    'gen_mac',
    'gen_netmask',
    'gen_negative_integer',
    'gen_numeric_string',
    'gen_positive_integer',
    'gen_string',
    'gen_time',
    'gen_url',
    'gen_utf8',
    'gen_uuid',
    'gen_html_with_total_len'
)


# Private Functions -----------------------------------------------------------


def _make_unicode(data):
    """Convert ``data`` to a unicode string if running Python 2.

    :param str data: A string to be type cast.
    :return: ``data``, but as unicode. ``data`` is never modified: if a type
        cast is necessary, a copy of ``data`` is returned.

    """
    if sys.version_info[0] == 2:
        return unicode(data)  # flake8:noqa pylint:disable=undefined-variable
    return data


def _is_positive_int(length):
    """Check that ``length`` argument is an integer greater than zero.

    :param int length: The desired length of the string
    :raises: ``ValueError`` if ``length`` is not an ``int`` or is less than 1.
    :returns: Nothing.
    :rtype: None

    """
    if not isinstance(length, int) or length <= 0:
        raise ValueError("{0} is an invalid 'length'.".format(length))


def _unicode_letters_generator():
    """Generates unicode characters in the letters category

    :return: a generator which will generates all unicode letters available

    """
    if sys.version_info[0] == 2:
        chr_function = unichr  # pylint:disable=undefined-variable
        range_function = xrange  # pylint:disable=undefined-variable
    else:
        chr_function = chr
        range_function = range  # pylint:disable=redefined-variable-type

    # Use sys.maxunicode instead of 0x10FFFF to avoid the exception below, in a
    # narrow Python build (before Python 3.3)
    # ValueError: unichr() arg not in range(0x10000) (narrow Python build)
    # For more information, read PEP 261.
    for i in range_function(sys.maxunicode):
        char = chr_function(i)
        if unicodedata.category(char).startswith('L'):
            yield char


UNICODE_LETTERS = [c for c in _unicode_letters_generator()]


def _check_validation(fcn):
    """Simple decorator to validate values generate by fcn accordingly to
    parameters `validator`, `default` and `tries`

    :param fcn: function to be enhanced
    :return: decorated function
    """

    @wraps(fcn)
    def validate(*args, **kwargs):
        validator = kwargs.get('validator')
        default = kwargs.get('default')
        tries = kwargs.get('tries', 10)
        if validator and default is None:
            raise ValueError('If "validator" param is defined, "default" '
                             'parameter must not be None')
        if validator is None:
            def validator_fcn(_):
                return True
        else:
            validator_fcn = validator

        if not callable(validator_fcn):
            def regex_validator(value):
                return re.match(validator, value)

            validator_fcn = regex_validator

        # Removing params related to validation but not fcn
        for key in ('validator', 'default', 'tries'):
            if key in kwargs:
                kwargs.pop(key)

        for _ in range(tries):
            value = fcn(*args, **kwargs)
            if validator_fcn(value):
                return value

        return default

    return validate


# Public Functions ------------------------------------------------------------


def gen_string(str_type, length=None, validator=None, default=None, tries=10):
    """A simple wrapper that calls other string generation methods.

    :param str str_type: The type of string which should be generated.
    :param int length: The length of the generated string. Must be 1 or
        greater.
    :param validator: Function or regex (str).
        If a function it must receive one parameter and return True if value
        can be used and False of another value need to be generated.
        If str it will be used as regex to validate the generated value.
        Default is None which will not validate the value.
    :param tries: number of times validator must be called before returning
        `default`. Default is 10.
    :param default: If validator returns false a number of `tries` times, this
        value is returned instead. Must be defined if validator is not None
    :raises: ``ValueError`` if an invalid ``str_type`` is specified.
    :returns: A string.
    :rtype: str

    Valid values for ``str_type`` are as follows:

    * alpha
    * alphanumeric
    * cjk
    * cyrillic
    * html
    * latin1
    * numeric
    * utf8

    """
    str_types_functions = {
        u'alpha': gen_alpha,
        u'alphanumeric': gen_alphanumeric,
        u'cjk': gen_cjk,
        u'cyrillic': gen_cyrillic,
        u'html': gen_html,
        u'latin1': gen_latin1,
        u'numeric': gen_numeric_string,
        u'utf8': gen_utf8,
    }
    str_type_lower = str_type.lower()  # do not modify user data
    if str_type_lower not in str_types_functions.keys():
        raise ValueError(
            '{0} is not a supported string type. Valid string types are {1}.'
            ''.format(str_type_lower, u','.join(str_types_functions.keys()))
        )
    method = str_types_functions[str_type_lower]
    if length is None:
        return method(validator=validator, default=default, tries=tries)
    return method(length, validator=validator, default=default, tries=tries)


@_check_validation
def gen_alpha(length=10):
    """Returns a random string made up of alpha characters.

    :param int length: Length for random data.
    :returns: A random string made up of alpha characters.
    :rtype: str

    """

    # Validate length argument
    _is_positive_int(length)

    random.seed()
    output_string = u''.join(
        random.choice(string.ascii_letters) for i in range(length)
    )

    return _make_unicode(output_string)


@_check_validation
def gen_alphanumeric(length=10):
    """Returns a random string made up of alpha and numeric characters.

    :param int length: Length for random data.
    :returns: A random string made up of alpha and numeric characters.
    :rtype: str

    """

    # Validate length argument
    _is_positive_int(length)

    random.seed()
    output_string = u''.join(
        random.choice(
            string.ascii_letters + string.digits
        ) for i in range(length))

    return _make_unicode(output_string)


def gen_boolean():
    """Returns a random Boolean value.

    :returns: A random Boolean value.
    :rtype: bool

    """

    choices = (True, False)

    return gen_choice(choices)


def gen_choice(choices):
    """Returns a random choice from the available choices.

    :param list choices: List of choices from which select a random value.
    :raises: ``ValueError`` if ``choices`` is ``None`` or not ``Iterable`` or
        a ``dict``.
    :returns: A random element from ``choices``.

    """

    # Validation for 'choices'
    if choices is None:
        raise ValueError("Choices argument cannot be None.")
    # We don't want a single dictionary value.
    if not isinstance(choices, Iterable) or isinstance(choices, dict):
        raise ValueError("Choices argument is not iterable.")
    if len(choices) == 0:
        raise ValueError("Choices argument cannot be empty.")
    # If only 1 item is present, return it right away
    if len(choices) == 1:
        return choices[0]

    random.seed()
    return random.choice(choices)


@_check_validation
def gen_cjk(length=10):
    """Returns a random string made up of CJK characters.
    (Source: Wikipedia - CJK Unified Ideographs)

    :param int length: Length for random data.
    :returns: A random string made up of CJK characters.
    :rtype: str

    """

    # Validate length argument
    _is_positive_int(length)

    random.seed()

    # Generate codepoints, then convert the codepoints to a string. The
    # valid range of CJK codepoints is 0x4E00 - 0x9FCC, inclusive. Python 2
    # and 3 support the `unichr` and `chr` functions, respectively.
    codepoints = [random.randint(0x4E00, 0x9FCC) for _ in range(length)]
    if sys.version_info[0] == 2:
        # pylint:disable=undefined-variable
        output = u''.join(unichr(codepoint) for codepoint in codepoints)
    else:
        output = u''.join(chr(codepoint) for codepoint in codepoints)
    return _make_unicode(output)


@_check_validation
def gen_cyrillic(length=10):
    """Returns a random string made up of Cyrillic characters.

    :param int length: Length for random data.
    :returns: A random string made up of Cyrillic characters.
    :rtype: str

    """

    # Validate length argument
    _is_positive_int(length)

    random.seed()

    # Generate codepoints, then convert the codepoints to a string. The
    # valid range of Cyrillic codepoints is 0x410 - 0x4ff, inclusive. Python 2
    # and 3 support the `unichr` and `chr` functions, respectively.
    codepoints = [random.randint(0x0400, 0x04FF) for _ in range(length)]
    try:
        # (undefined-variable) pylint:disable=E0602
        output = u''.join(unichr(codepoint) for codepoint in codepoints)
    except NameError:
        output = u''.join(chr(codepoint) for codepoint in codepoints)
    return _make_unicode(output)


def gen_date(min_date=None, max_date=None):
    """Returns a random date value

    :param min_date: A valid ``datetime.date`` object.
    :param max_date: A valid ``datetime.date`` object.
    :raises: ``ValueError`` if arguments are not valid ``datetime.date``
        objects.
    :returns: Random ``datetime.date`` object.

    """

    _min_value = (datetime.date.today() -
                  datetime.timedelta(365 * MIN_YEARS))
    _max_value = (datetime.date.today() +
                  datetime.timedelta(365 * MAX_YEARS))

    if min_date is None:
        min_date = _min_value
    if max_date is None:
        max_date = _max_value

    # Validation
    if not isinstance(min_date, datetime.date):
        raise ValueError("%s is not a valid datetime.date object")
    if not isinstance(max_date, datetime.date):
        raise ValueError("%s is not a valid datetime.date object")

    # Check that max_date is not before min_date
    assert min_date < max_date

    random.seed()

    # Pick a day between min and max dates
    diff = max_date - min_date
    days = random.randint(0, diff.days)
    date = min_date + datetime.timedelta(days=days)

    return date


def gen_datetime(min_date=None, max_date=None):
    """Returns a random datetime value

    :param min_date: A valid ``datetime.datetime`` object.
    :param max_date: A valid ``datetime.datetime`` object.
    :raises: ``ValueError`` if arguments are not valid ``datetime.datetime``
        objects.
    :returns: Random ``datetime.datetime`` object.

    """

    _min_value = (datetime.datetime.now() -
                  datetime.timedelta(365 * MIN_YEARS))
    _max_value = (datetime.datetime.now() +
                  datetime.timedelta(365 * MAX_YEARS))

    if min_date is None:
        min_date = _min_value
    if max_date is None:
        max_date = _max_value

    # Validation
    if not isinstance(min_date, datetime.datetime):
        raise ValueError("%s is not a valid datetime.datetime object")
    if not isinstance(max_date, datetime.datetime):
        raise ValueError("%s is not a valid datetime.datetime object")

    # Check that max_date is not before min_date
    assert min_date < max_date

    random.seed()

    # Pick a time between min and max dates
    diff = max_date - min_date
    seconds = random.randint(0, diff.days * 3600 * 24 + diff.seconds)

    return min_date + datetime.timedelta(seconds=seconds)


@_check_validation
def gen_email(name=None, domain=None, tlds=None):
    """Generates a random email address.

    :param str name: Email name.
    :param str domain: Domain name.
    :param str tlds: Top Level Domain Server
    :returns: An email address.
    :rtype: str

    """

    # Generate a new name if needed
    if name is None:
        name = gen_alpha(8)
    # Obtain a random domain if needed
    if domain is None:
        domain = gen_choice(SUBDOMAINS)
    # Obtain a random top level domain if needed
    if tlds is None:
        tlds = gen_choice(TLDS)

    email = u"{0}@{1}.{2}".format(name, domain, tlds)

    return _make_unicode(email)


def gen_integer(min_value=None, max_value=None):
    """Returns a random integer value based on the current platform.

    :param int min_value: The minimum allowed value.
    :param int max_value: The maximum allowed value.
    :raises: ``ValueError`` if arguments are not integers or if they are
        less or greater than the system's allowed range for integers.
    :returns: Returns a random integer value.
    :rtype: int

    """

    # Platform-specific value range for integers
    _min_value = - sys.maxsize - 1
    _max_value = sys.maxsize

    if min_value is None:
        min_value = _min_value
    if max_value is None:
        max_value = _max_value

    if sys.version_info[0] < 3:
        integer_types = (int, long,)  # pylint:disable=undefined-variable
    else:
        integer_types = (int,)

    # Perform some validations
    if not isinstance(min_value, integer_types) or min_value < _min_value:
        raise ValueError("\'%s\' is not a valid minimum." % min_value)
    if not isinstance(max_value, integer_types) or max_value > _max_value:
        raise ValueError("\'%s\' is not a valid maximum." % max_value)

    random.seed()
    value = random.randint(min_value, max_value)

    return value


def gen_iplum(words=None, paragraphs=None):
    """Returns a lorem ipsum string. If no arguments are passed, then
    return the entire default lorem ipsum string.

    :param int words: The number of words to return.
    :param int paragraphs: The number of paragraphs to return.
    :raises: ``ValueError`` if ``words`` is not a valid positive integer.
    :returns: A ``lorem ipsum`` string containing either the number of
    ``words``
        or ``paragraphs``, extending and wrapping around the text as needed to
        make sure that it has the specified length.
    :rtype: str

    """

    # Check parameters
    if words is None or words == 0:
        words = len(LOREM_IPSUM_TEXT.split())
    if paragraphs is None:
        paragraphs = 1

    if not isinstance(words, int) or words < 0:
        raise ValueError(
            "Cannot generate a string with negative number of words.")
    _is_positive_int(paragraphs)

    # Original Lorem Ipsum string
    all_words = LOREM_IPSUM_TEXT.split()
    # How many words do we need?
    total_words_needed = words * paragraphs

    quotient = int(total_words_needed / len(all_words))
    modulus = total_words_needed % len(all_words)

    # Pool of words to use
    all_words = all_words * (quotient + modulus)

    result = u""
    start_pos = 0
    for _ in range(0, paragraphs):
        sentence = u" ".join(
            all_words[start_pos:start_pos + words])

        # Remove comma from the end, if it exists
        if sentence.endswith(','):
            sentence = sentence.rstrip(',')
        # Remove period from the end, if it exists
        if sentence.endswith('.'):
            sentence = sentence.rstrip('.')

        # Each sentence should be properly capitalized
        cap_sentence = [
            frag.capitalize() + u'.' for frag in sentence.split('. ')]

        # Add newline at the end
        result += " ".join(cap_sentence) + u"\n"

        # Increment positional counter
        start_pos += words
    return _make_unicode(result.rstrip())


@_check_validation
def gen_latin1(length=10):
    """Returns a random string made up of UTF-8 characters.
    (Font: Wikipedia - Latin-1 Supplement Unicode Block)

    :param int length: Length for random data.
    :returns: A random string made up of ``Latin1`` characters.
    :rtype: str

    """

    # Validate length argument
    _is_positive_int(length)

    range0 = range1 = range2 = []
    range0 = ['00C0', '00D6']
    range1 = ['00D8', '00F6']
    range2 = ['00F8', '00FF']
    output_array = []

    for i in range(int(range0[0], 16), int(range0[1], 16)):
        output_array.append(i)
    for i in range(int(range1[0], 16), int(range1[1], 16)):
        output_array.append(i)
    for i in range(int(range2[0], 16), int(range2[1], 16)):
        output_array.append(i)

    random.seed()

    if sys.version_info[0] == 2:
        output_string = u''.join(
            # pylint:disable=E0602
            unichr(random.choice(output_array)) for _ in range(length)
        )
    else:
        output_string = u''.join(
            chr(random.choice(output_array)) for _ in range(length)
        )

    return _make_unicode(output_string)


def gen_negative_integer():
    """Returns a random negative integer based on the current platform.

    :returns: Returns a random negative integer value.
    :rtype: int

    """

    max_value = 0

    return gen_integer(max_value=max_value)


@_check_validation
def gen_ipaddr(ip3=False, ipv6=False, prefix=()):
    """Generates a random IP address.
    You can also specify an IP address prefix if you are interested in
    local network address generation, etc.

    :param bool ip3: Whether to generate a 3 or 4 group IP.
    :param bool ipv6: Whether to generate IPv6 or IPv4
    :param list prefix: A prefix to be used for an IP (e.g. [10, 0, 1]). It
        must be an iterable with strings or integers. Can be left
        unspecified or empty.
    :returns: An IP address.
    :rtype: str
    :raises: ``ValueError`` if ``prefix`` would lead to no random fields at
    all.
        This means the length that triggers the ``ValueError`` is 4 for regular
        IPv4, 3 for IPv4 with ip3 and 8 for IPv6. It will be raised in any case
        the prefix length reaches or exceeds those values.

    """

    # Set the lengths of the randomly generated sections
    if ipv6:
        rng = 8
    elif ip3:
        rng = 3
    else:
        rng = 4
    prefix = [str(field) for field in prefix]
    # Prefix reduces number of random fields generated, so subtract the length
    # of it from the rng to keep the IP address have correct number of fields
    rng -= len(prefix)
    if rng == 0:
        raise ValueError(
            "Prefix {} would lead to no randomness at all".format(
                repr(prefix)))
    elif rng < 0:
        raise ValueError(
            "Prefix {} is too long for this configuration".format(
                repr(prefix)))

    random.seed()

    if ipv6:
        # StackOverflow.com questions: generate-random-ipv6-address
        random_fields = [
            '{0:x}'.format(random.randint(0, 2 ** 16 - 1)) for _ in range(rng)]
        ipaddr = u':'.join(prefix + random_fields)
    else:
        random_fields = [str(random.randrange(0, 255, 1)) for _ in range(rng)]
        ipaddr = u".".join(prefix + random_fields)

        if ip3:
            ipaddr = ipaddr + u".0"

    return _make_unicode(ipaddr)


@_check_validation
def gen_mac(delimiter=':', multicast=None, locally=None):
    """Generates a random MAC address.

    For more information about how unicast or multicast and globally unique and
    locally administered MAC addresses are generated check this link
    https://en.wikipedia.org/wiki/MAC_address.

    :param str delimeter: Valid MAC delimeter (e.g ':', '-').
    :param bool multicast: Indicates if the generated MAC address should be
        unicast or multicast. If no value is provided a random one will be
        chosen.
    :param bool locally: Indicates if the generated MAC address should be
        globally unique or locally administered. If no value is provided a
        random one will be chosen.
    :returns: A random MAC address.
    :rtype: str

    """

    if delimiter not in [':', '-']:
        raise ValueError('Delimiter is not a valid option: %s' % delimiter)
    random.seed()
    if multicast is None:
        multicast = bool(random.randint(0, 1))
    if locally is None:
        locally = bool(random.randint(0, 1))

    first_octet = random.randint(0, 255)
    if multicast:
        # Ensure that the first least significant bit is 1
        first_octet |= 0b00000001
    else:
        # Ensure that the first least significant bit is 0
        first_octet &= 0b11111110
    if locally:
        # Ensure that the second least significant bit is 1
        first_octet |= 0b00000010
    else:
        # Ensure that the second least significant bit is 0
        first_octet &= 0b11111101

    octets = [first_octet]
    octets.extend(random.randint(0, 255) for _ in range(5))
    mac = delimiter.join(['{0:02x}'.format(octet) for octet in octets])

    return _make_unicode(mac)


@_check_validation
def gen_netmask(min_cidr=1, max_cidr=31):
    """Generates a random valid netmask.

    For more info: http://www.iplocation.net/tools/netmask.php

    :param int min_cidr: Inferior CIDR limit
    :param int max_cidr: Superior CIDR limit
    :returns: The netmask is chosen from
        :data:`fauxfactory.constants.VALID_NETMASKS` respecting the CIDR range
    :rtype: str
    :raises: ``ValueError`` if ``min_cidr`` or ``max_cidr`` have an invalid
        value. For example, ``max_cidr`` cannot be 33.

    """
    if min_cidr < 0:
        raise ValueError(
            'min_cidr must be 0 or greater, but is {0}'.format(min_cidr)
        )
    if max_cidr >= len(VALID_NETMASKS):
        raise ValueError(
            'max_cidr must be less than {0}, but is {1}'
                .format(len(VALID_NETMASKS), max_cidr)
        )
    random.seed()
    return VALID_NETMASKS[random.randint(min_cidr, max_cidr)]


@_check_validation
def gen_numeric_string(length=10):
    """Returns a random string made up of numbers.

    :param int length: Length for random data.
    :returns: A random string made up of numbers.
    :rtype: str

    """

    # Validate length argument
    _is_positive_int(length)

    random.seed()
    output_string = u''.join(
        random.choice(string.digits) for i in range(length)
    )

    return _make_unicode(output_string)


def gen_positive_integer():
    """Returns a random positive integer based on the current platform.

    :returns: A random positive integer value.
    :rtype: int

    """

    min_value = 0

    return gen_integer(min_value=min_value)


def gen_time():
    """Generates a random time.

    :returns: A random ``datetime.time`` object.

    """

    random.seed()
    return datetime.time(
        random.randint(0, 23),
        random.randint(0, 59),
        random.randint(0, 59),
        random.randint(0, 999999),
    )


@_check_validation
def gen_url(scheme=None, subdomain=None, tlds=None):
    """Generates a random URL address

    :param str scheme: Either http, https or ftp.
    :param str subdomain: A valid subdmain
    :param str tlds: A qualified top level domain name (e.g. 'com', 'net')
    :raises: ``ValueError`` if arguments are not valid.
    :returns: A random URL address.
    :rtype: str

    """

    # Regex for subdomain names
    subdomainator = re.compile(r"^[a-zA-Z0-9][-\w.~]*$")
    # Regex for URL scheme
    schemenator = re.compile(r"^(https?|ftp)$")
    # Regex for TLDS
    tldsnator = re.compile(r"^[a-zA-Z]{1,3}$")

    if scheme:
        if schemenator.match(scheme) is None:
            raise ValueError("Protocol {0} is not valid.".format(scheme))
    else:
        scheme = gen_choice(SCHEMES)

    if subdomain:
        if subdomainator.match(subdomain) is None:
            raise ValueError("Subdomain {0} is invalid.".format(subdomain))
    else:
        subdomain = gen_choice(SUBDOMAINS)

    if tlds:
        if tldsnator.match(tlds) is None:
            raise ValueError("TLDS name {0} is invalid.".format(tlds))
    else:
        tlds = gen_choice(TLDS)

    url = u"{0}://{1}.{2}".format(scheme, subdomain, tlds)

    return _make_unicode(url)


@_check_validation
def gen_utf8(length=10):
    """Returns a random string made up of UTF-8 letters characters, as per
    `RFC 3629`_.

    :param int length: Length for random data.
    :returns: A random string made up of ``UTF-8`` letters characters.
    :rtype: str

    .. _`RFC 3629`: http://www.rfc-editor.org/rfc/rfc3629.txt

    """
    # Validate length argument
    _is_positive_int(length)

    random.seed()
    return u''.join([random.choice(UNICODE_LETTERS) for _ in range(length)])


@_check_validation
def gen_uuid():
    """Generates a UUID string (universally unique identifiers).

    :returns: Returns a string representation for a UUID.
    :rtype: str

    """

    output_uuid = _make_unicode(str(uuid.uuid4()))

    return output_uuid


@_check_validation
def gen_html(length=10):
    """Returns a random string made up of html characters.

    :param int length: Length for random data.
    :returns: A random string made up of html characters.
    :rtype: str

    """

    # Validate length argument
    _is_positive_int(length)

    random.seed()
    html_tag = random.choice(HTML_TAGS)
    output_string = u'<{0}>{1}</{2}>'.format(
        html_tag, gen_string("alpha", length), html_tag)

    return _make_unicode(output_string)


@_check_validation
def gen_html_with_total_len(length=10):
    """Returns a random string made up of html characters.
    This differ from fauxfactory.gen_html because length takes html tag chars
    into account
    :param int length: Length for random data.
    :returns: A random string made up of html characters.
    :rtype: str

    """
    if length < 8:
        raise ValueError(u'Impossible to generate html with less than 7 chars')

    random.seed()
    html_tag = random.choice(HTML_TAGS)
    maybe_len = length - len('<{0}></{0}>'.format(html_tag))
    if maybe_len <= 0:
        length -= 7
        html_tag = 'a'
    else:
        length = maybe_len
    output_string = u'<{0}>{1}</{2}>'.format(
        html_tag, gen_string("alpha", length), html_tag)

    return _make_unicode(output_string)


# Backward Compatibility ------------------------------------------------------


# Code borrowed from http://code.activestate.com/recipes/391367-deprecated/
def deprecated(func):
    """A decorator used to mark functions as deprecated.

    Emit a warning when the decorated function is called.

    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        """Emit a warning, then call ``func``."""
        old_name = func.__name__
        if old_name == 'codify':
            new_name = '_make_unicode'
        else:
            new_name = old_name.replace('generate', 'gen')
        warnings.warn(
            '{0} is deprecated! Please use {1} instead.'
                .format(old_name, new_name),
            category=Warning
        )
        return func(*args, **kwargs)

    return wrapper


@deprecated
def codify(data):
    # pylint:disable=missing-docstring
    return _make_unicode(data)


class FauxFactory(object):
    # This issue is no longer relevant, as the class has been turned into a set
    # of functions.
    # pylint:disable=too-many-public-methods
    #
    # This code is not imported when `from fauxfactory import *` is called, nor
    # does this code show up in Sphinx's output. See `__all__`.
    # pylint:disable=missing-docstring

    @classmethod
    @deprecated
    def generate_string(cls, str_type, length):
        return gen_string(str_type, length)

    @classmethod
    @deprecated
    def generate_alpha(cls, length=10):
        return gen_alpha(length)

    @classmethod
    @deprecated
    def generate_alphanumeric(cls, length=10):
        return gen_alphanumeric(length)

    @classmethod
    @deprecated
    def generate_boolean(cls):
        return gen_boolean()

    @classmethod
    @deprecated
    def generate_choice(cls, choices):
        return gen_choice(choices)

    @classmethod
    @deprecated
    def generate_cjk(cls, length=10):
        return gen_cjk(length)

    @classmethod
    @deprecated
    def generate_date(cls, min_date=None, max_date=None):
        return gen_date(min_date, max_date)

    @classmethod
    @deprecated
    def generate_datetime(cls, min_date=None, max_date=None):
        return gen_datetime(min_date, max_date)

    @classmethod
    @deprecated
    def generate_email(cls, name=None, domain=None, tlds=None):
        return gen_email(name, domain, tlds)

    @classmethod
    @deprecated
    def generate_integer(cls, min_value=None, max_value=None):
        return gen_integer(min_value, max_value)

    @classmethod
    @deprecated
    def generate_iplum(cls, words=None, paragraphs=None):
        return gen_iplum(words, paragraphs)

    @classmethod
    @deprecated
    def generate_latin1(cls, length=10):
        return gen_latin1(length)

    @classmethod
    @deprecated
    def generate_negative_integer(cls):
        return gen_negative_integer()

    @classmethod
    @deprecated
    def generate_ipaddr(cls, ip3=False, ipv6=False):
        return gen_ipaddr(ip3, ipv6)

    @classmethod
    @deprecated
    def generate_mac(cls, delimiter=":"):
        return gen_mac(delimiter)

    @classmethod
    @deprecated
    def generate_numeric_string(cls, length=10):
        return gen_numeric_string(length)

    @classmethod
    @deprecated
    def generate_positive_integer(cls):
        return gen_integer()

    @classmethod
    @deprecated
    def generate_time(cls):
        return gen_time()

    @classmethod
    @deprecated
    def generate_url(cls, scheme=None, subdomain=None, tlds=None):
        return gen_url(scheme, subdomain, tlds)

    @classmethod
    @deprecated
    def generate_utf8(cls, length=10):
        return gen_utf8(length)

    @classmethod
    @deprecated
    def generate_uuid(cls):
        return gen_uuid()

    @classmethod
    @deprecated
    def generate_html(cls, length=10):
        return gen_html(length)



# -*- coding: utf-8 -*-

"""Tests for MAC generator."""

import random
import re
import string
from sys import version_info
from fauxfactory import gen_mac

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


mac = re.compile("[0-9a-f]{2}([-:])[0-9a-f]{2}(\\1[0-9a-f]{2}){4}$")


class TestMacs(unittest.TestCase):
    """Test MAC generator."""

    def test_gen_mac_1(self):
        """
        @Test: Generate a MAC address using \":\" as the delimiter
        @Feature: MAC Generator
        @Assert: A MAC address is generated using \":\" as the delimiter
        """

        result = gen_mac()
        self.assertTrue(
            len(result.split(":")) == 6,
            "Did not generate a MAC addrss")
        self.assertIsNotNone(
            mac.match(result),
            "Did not match regular expression for MAC address")

    def test_gen_mac_2(self):
        """
        @Test: Generate a MAC address using \":\" as the delimiter
        @Feature: MAC Generator
        @Assert: A MAC address is generated using \":\" as the delimiter
        """

        result = gen_mac(delimiter=":")
        self.assertTrue(
            len(result.split(":")) == 6,
            "Did not generate a MAC addrss")
        self.assertIsNotNone(
            mac.match(result),
            "Did not match regular expression for MAC address")

    def test_gen_mac_3(self):
        """
        @Test: Generate a MAC address using \"-\" as the delimiter
        @Feature: MAC Generator
        @Assert: A MAC address is generated using \"-\" as the delimiter
        """

        result = gen_mac(delimiter="-")
        self.assertTrue(
            len(result.split("-")) == 6,
            "Did not generate a MAC addrss")
        self.assertIsNotNone(
            mac.match(result),
            "Did not match regular expression for MAC address")

    def test_gen_mac_4(self):
        """
        @Test: Generate a MAC address using \".\" as the delimiter
        @Feature: MAC Generator
        @Assert: A MAC address is not generated using an invalid delimiter
        """

        with self.assertRaises(ValueError):
            gen_mac(delimiter=".")

    def test_gen_mac_5(self):
        """
        @Test: Generate a MAC address using \" \" as the delimiter
        @Feature: MAC Generator
        @Assert: A MAC address is not generated using an invalid delimiter
        """

        with self.assertRaises(ValueError):
            gen_mac(delimiter=" ")

    def test_gen_mac_6(self):
        """
        @Test: Generate a MAC address using a number as the delimiter
        @Feature: MAC Generator
        @Assert: A MAC address is not generated using an invalid delimiter
        """

        with self.assertRaises(ValueError):
            gen_mac(delimiter=random.randint(0, 10))

    def test_gen_mac_7(self):
        """
        @Test: Generate a MAC address using a letter as the delimiter
        @Feature: MAC Generator
        @Assert: A MAC address is not generated using an invalid delimiter
        """

        with self.assertRaises(ValueError):
            gen_mac(
                delimiter=random.choice(string.ascii_letters))

    def test_gen_mac_unicast_globally_unique(self):
        """
        @Test: Generate a unicast and globally unique MAC address
        @Feature: MAC Generator
        @Assert: A unicast and globally unique MAC address is generated
        """
        mac = gen_mac(multicast=False, locally=False)
        first_octect = int(mac.split(':', 1)[0], 16)
        mask = 0b00000011
        self.assertEqual(first_octect & mask, 0)

    def test_gen_mac_multicast_globally_unique(self):
        """
        @Test: Generate a multicast and globally unique MAC address
        @Feature: MAC Generator
        @Assert: A multicast and globally unique MAC address is generated
        """
        mac = gen_mac(multicast=True, locally=False)
        first_octect = int(mac.split(':', 1)[0], 16)
        mask = 0b00000011
        self.assertEqual(first_octect & mask, 1)

    def test_gen_mac_unicast_locally_administered(self):
        """
        @Test: Generate a unicast and locally administered MAC address
        @Feature: MAC Generator
        @Assert: A unicast and locally administered MAC address is generated
        """
        mac = gen_mac(multicast=False, locally=True)
        first_octect = int(mac.split(':', 1)[0], 16)
        mask = 0b00000011
        self.assertEqual(first_octect & mask, 2)

    def test_gen_mac_multicast_locally_administered(self):
        """
        @Test: Generate a multicast and locally administered MAC address
        @Feature: MAC Generator
        @Assert: A multicast and locally administered MAC address is generated
        """
        mac = gen_mac(multicast=True, locally=True)
        first_octect = int(mac.split(':', 1)[0], 16)
        mask = 0b00000011
        self.assertEqual(first_octect & mask, 3)



# -*- coding: utf-8 -*-

"""Tests for Time generator."""

import datetime
from sys import version_info
from fauxfactory import gen_time

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestTime(unittest.TestCase):
    """Test Time generator."""

    def test_gen_uuid_1(self):
        """
        @Test: Create a random UUID value
        @Feature: UUID Generator
        @Assert: A random UUID value is generated
        """

        for turn in range(100):
            result = gen_time()
            self.assertIsInstance(
                result, datetime.time,
                "A valid Time value was not generated.")



# -*- coding: utf-8 -*-

"""Tests for HTML generator."""

from sys import version_info
from fauxfactory import gen_html, gen_integer, gen_html_with_total_len
import re
# (too-many-public-methods) pylint:disable=R0904

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestHTML(unittest.TestCase):
    """Test HTML generator."""

    @classmethod
    def setUpClass(cls):
        """Instantiate a factory and compile a regex.

        The compiled regex can be used to find the contents of an HTML tag.

        """
        cls.matcher = re.compile('^<.*?>(.*?)</.*>$')

    def test_length_arg_omitted(self):
        """
        @Test: Generate a random HTML tag and provide no value for the
            ``length`` argument.
        @Feature: HTML Generator
        @Assert: The contents of the HTML tag are at least one character long.
        """

        match = self.matcher.search(gen_html())
        self.assertGreaterEqual(len(match.group(1)), 1)

    def test_length_arg_provided(self):
        """
        @Test: Generate a random HTML tag and provide a value for the
            ``length`` argument.
        @Feature: HTML Generator
        @Assert: The contents of the HTML tag are ``length`` characters long.
        """

        length = gen_integer(1, 25)
        match = self.matcher.search(gen_html(length))
        self.assertEqual(len(match.group(1)), length)

    def test_unicode(self):
        """
        @Test: Generate a random HTML tag.
        @Feature: HTML Generator
        @Assert: A unicode string is generated.
        """

        result = gen_html()
        if version_info[0] is 2:
            # (undefined-variable) pylint:disable=E0602
            self.assertIsInstance(result, unicode)  # flake8:noqa
        else:
            self.assertIsInstance(result, str)

    def test_generate_html_with_len_less_than_min(self):
        for i in range(8):
            self.assertRaises(ValueError, gen_html_with_total_len, i)

    def test_generate_html_with_len_more_than_min(self):
        length = gen_integer(8, 25)
        self.assertEqual(length, len(gen_html_with_total_len(length)))



"""Tests to help ensure that FauxFactory remains backward compatible."""
from fauxfactory import codify, FauxFactory
from functools import partial
from sys import version_info

import datetime
import warnings
# (too-many-public-methods) pylint:disable=R0904

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


GENERATORS = (
    partial(codify, 'make-me-unicode'),
    partial(FauxFactory.generate_string, 'alpha', 3),
    partial(FauxFactory.generate_alpha, 3),
    partial(FauxFactory.generate_alphanumeric, 3),
    FauxFactory.generate_boolean,
    partial(FauxFactory.generate_choice, (1, 2)),
    partial(FauxFactory.generate_cjk, 3),
    partial(
        FauxFactory.generate_date,
        datetime.date.today() - datetime.timedelta(1),
        datetime.date.today()
    ),
    partial(
        FauxFactory.generate_datetime,
        datetime.datetime.now() - datetime.timedelta(1),
        datetime.datetime.today()
    ),
    partial(FauxFactory.generate_email, 'Alice', 'example', 'com'),
    partial(FauxFactory.generate_integer, 1, 10),
    partial(FauxFactory.generate_iplum, 1, 1),
    partial(FauxFactory.generate_latin1, 3),
    FauxFactory.generate_negative_integer,
    partial(FauxFactory.generate_ipaddr, True, True),
    partial(FauxFactory.generate_mac, '-'),
    partial(FauxFactory.generate_numeric_string, 3),
    FauxFactory.generate_positive_integer,
    FauxFactory.generate_time,
    partial(FauxFactory.generate_url, 'ftp', 'example', 'com'),
    partial(FauxFactory.generate_utf8, 3),
    FauxFactory.generate_uuid,
    partial(FauxFactory.generate_html, 3),
)


class FauxFactoryTestCase(unittest.TestCase):
    """Call methods on class ``FauxFactory``."""
    def test_not_none(self):
        """Call every method on ``FauxFactory`` and provide arguments.

        This test does very little: it simply calls each method on
        ``FauxFactory``, provides a value for each possible argument, and
        asserts that the return value is not ``None``. This is not amazingly
        useful. However, it does provide a basic sanity check. It tells us that
        each wrapper method:

        * can run without raising an exception,
        * can accept an appropriate number of arguments, and
        * returns an "interesting" value.

        """

        for generator in GENERATORS:
            self.assertIsNotNone(generator())

    def test_deprecated_warning(self):
        """Check that ``FauxFactory`` functions show deprecation message.

        """

        for generator in GENERATORS:
            with warnings.catch_warnings(record=True) as faux_warn:
                self.assertIsNotNone(generator())
                self.assertIn('deprecated', str(faux_warn[-1].message))



# -*- coding: utf-8 -*-
"""Tests for the netmask generator"""

import re
from sys import version_info
from fauxfactory import gen_netmask, VALID_NETMASKS

if version_info[0:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest


NETMASK_REGEX = re.compile(
    '((255.){3}(0|128|192|224|240|248|252|254|255))|'
    '((255.){2}(0|128|192|224|240|248|252|254).0)|'
    '(255.(0|128|192|224|240|248|252|254)(.0){2})|'
    '((0|128|192|224|240|248|252|254)(.0){3})'
)


class NetmaskTestCase(unittest.TestCase):
    """Tests for ``gen_netmask`` generator."""

    def test_gen_netmask(self):
        """Test if gen_netmask generates valid values"""
        result = gen_netmask()
        self.assertEqual(len(result.split('.')), 4)
        self.assertIsNotNone(NETMASK_REGEX.match(result))

    def test_gen_netmask_boundary(self):
        """Test gen_netmask boundary cases"""
        self.assertEqual(u'0.0.0.0', gen_netmask(0, 0))
        self.assertEqual(u'255.255.255.255', gen_netmask(32, 32))
        with self.assertRaises(ValueError):
            gen_netmask(-1, 16)
        with self.assertRaises(ValueError):
            gen_netmask(16, 33)

    def test_valid_netmasks(self):
        """Test if VALID_NETMASKS constant have valid netmask values"""
        for netmask in VALID_NETMASKS:
            self.assertIsNotNone(NETMASK_REGEX.match(netmask))
